
# q-expansion of newform 5040.2.a.be, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5040
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [1], [-5], [1], [-3], [6], [-6], [9], [0], [6], [-8], [-6], [3], [12], [8], [-4], [4], [8], [10], [3], [-12], [16], [7], [14], [9], [2], [-11], [6], [-8], [-6], [16], [-18], [14], [19], [-2], [-6], [-9], [-19], [4], [-20], [11], [8], [12], [0], [-13], [25], [-13], [16], [-8], [-7], [18], [14], [-6], [18], [-10], [-4], [14], [13], [-29], [1], [27], [-14], [29], [-30], [20], [-26], [-10], [14], [33], [0], [29], [0], [20], [12], [-25], [-29], [-9], [14], [36], [-19], [23], [-30], [34], [30], [33], [-28], [24], [-20], [33], [-6], [-26], [33], [25], [-31], [-18], [6], [-4], [-9], [-20], [-24], [4], [-34], [36], [17], [8], [7], [-33], [-34], [-17], [-22], [-10], [-2], [9], [-34], [-47], [0], [-30], [-25], [8], [32], [-33], [-20], [-20], [21], [-41], [50], [16], [5], [37], [-36], [35], [8], [46], [-10], [-1], [-11], [-35], [23], [-38], [7], [-32], [2], [16], [-26], [-26], [14], [28], [20], [-10], [16], [16], [-8], [30], [-40], [-25], [-36], [7], [-4], [-52], [-20], [-22], [28], [-18], [9], [16], [-1], [37], [-18], [-24], [10], [27], [-30], [-10], [-36], [9], [15], [-60], [-28], [8], [0], [-6], [-6], [-40], [-14], [-16], [25], [25], [-12], [66], [6], [-17], [44], [-28], [15], [-1], [-32], [61], [23], [18], [10], [10], [-2], [6], [33], [10], [-32], [-22], [12], [-8], [-26], [0], [3], [6], [48], [14], [-40], [-46], [-54], [-62], [74], [-45], [18], [-53], [-66], [-55], [-9], [17], [-28], [24], [-42], [-72], [63], [37], [-73], [20], [40], [-23], [-6], [24], [32], [-59], [54], [-9], [-2], [-32], [-18], [19], [-66], [-50], [38], [52], [-16], [18], [-9], [46], [-4], [-33], [-63], [18], [78], [60], [61], [-42], [-50], [-1], [14], [36], [-38], [60], [-8], [55], [-28], [-2], [53], [-12], [-30], [-31], [46], [-25], [-13], [32], [38], [-3], [-33], [50], [18], [60], [12], [-33], [-47], [36], [-16], [-10], [84], [26], [-49], [-36], [57], [-53], [-62], [85], [-32], [15], [-50], [-34], [-48], [-48], [-32], [0], [21], [34], [-26], [78], [-56], [-26], [23], [-37], [-45], [-2], [44], [-26], [-8], [-62], [20], [-81], [-4], [-52], [32], [70], [16], [46], [29], [12], [-57], [13], [14], [66], [77], [-38], [-24], [50], [-51], [-66], [12], [38], [-56], [-14], [-3], [-88], [4], [-25], [-11], [25], [10], [18], [78], [82], [86], [28], [72], [84], [7], [36], [38], [13], [70], [69], [-65], [76], [24], [-27], [-10], [26], [48], [-5], [9], [-54], [21], [18], [66], [68], [34], [44], [31], [25], [17], [-4], [-19], [-55], [-23], [-51], [68], [23], [-64], [16], [-76], [13], [-50], [0], [44], [94], [-7], [36], [26], [-86], [-15], [76], [52], [-65], [50], [54], [36], [58], [24], [63], [-84], [-2], [15], [67], [-90], [-74], [-12], [69], [-19], [56], [-66], [-28], [60]]
